using Amazon.Lambda.Core;
using Amazon;
using Amazon.SecretsManager;
using Amazon.SecretsManager.Model;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace LambdaSecretsManagerVPC;

public class Function
{
    public async Task<string> FunctionHandler(ILambdaContext context)
    {
        string secretName = "my-credentials";
        string region = "us-east-1";

        var timeout = context.RemainingTime.Subtract(TimeSpan.FromSeconds(3));
        var cts = new CancellationTokenSource(timeout);

        return await GetSecretValue(secretName, region, cts.Token);
    }

    private async Task<string> GetSecretValue(string secretName, string region, CancellationToken cancellationToken)
    {
        IAmazonSecretsManager client = new AmazonSecretsManagerClient(RegionEndpoint.GetBySystemName(region));

        GetSecretValueRequest request = new GetSecretValueRequest(){
            SecretId = secretName,
            VersionStage = "AWSCURRENT"
        };

        try
        {
            GetSecretValueResponse response = await client.GetSecretValueAsync(request, cancellationToken);
            return response.SecretString;
        }
        catch (OperationCanceledException oce)
        {
            System.Console.WriteLine($"Request timed out, the cancellation token was triggered. {oce.Message}");
            throw;
        }
        catch (Exception ex)
        {
            Console.Error.WriteLine("Error: " + ex.Message);
            throw;
        }
    }
}
